<?php

namespace Drupal\gdpr_tag_manager\Controller;

use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Utility\Error;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\GuzzleException;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * Class ContinentController.
 *
 * @package Drupal\gdpr_tag_manager\Controller
 */
class ContinentController extends ControllerBase {

  /**
   * The Guzzle HTTP client.
   *
   * @var \GuzzleHttp\Client
   */
  protected $client;

  /**
   * A logger instance.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * ImmutableConfig instance.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * ContinentController constructor.
   *
   * @param \GuzzleHttp\Client $client
   *   A guzzle http client instance.
   * @param \Psr\Log\LoggerInterface $logger
   *   The logger for this channel.
   * @param \Drupal\Core\Config\ImmutableConfig $config
   *   Config object for gdpr_tag_manager.settings.
   */
  public function __construct(Client $client, LoggerInterface $logger, ImmutableConfig $config) {
    $this->client = $client;
    $this->logger = $logger;
    $this->config = $config;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $client = $container->get('http_client');
    $logger = $container->get('logger.factory')
      ->get('gdpr_tag_manager_get_country_code');
    $config = $container->get('config.factory')
      ->get('gdpr_tag_manager.settings');
    return new static($client, $logger, $config);
  }

  /**
   * Page callback to retrieve continent code in Json.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   JSON response with properties 'c_code' and 'isanon'.
   */
  public function getContinentCode() {
    if (!empty($_SERVER["HTTP_CLIENT_IP"])){
     //check for ip from share internet
     $ip = $_SERVER["HTTP_CLIENT_IP"];
    }
    elseif (!empty($_SERVER["HTTP_X_FORWARDED_FOR"])){
     // Check for the Proxy User
     $ip = $_SERVER["HTTP_X_FORWARDED_FOR"];
    }
    else{
     $ip = $_SERVER["REMOTE_ADDR"];
    }
    $ip = str_replace(' ', '', end(array_values(array_filter(explode(',', $ip)))));
    $ip_service = $this->config->get('ip_service');
    $is_bot = $this::isBot($ip);
    $ip_data['isanon'] = \Drupal::currentUser()->isAnonymous() ? TRUE : FALSE;
    if ($this->config->get('activate') == 1 && $is_bot == FALSE) {
      $ip_data['c_code'] = $this->callGeoService($ip, $ip_service);
      return JsonResponse::create($ip_data);
    }
    else {
      return '';
    }
  }

  /**
   * Get continent code.
   *
   * @param string $ip
   *   Users ip address.
   * @param string $ip_service
   *   Plugin service to use. Expected values are 'IPAPI' or 'GEOIP'.
   *
   * @return array|string
   *   Continent code or empty array if unsuccessful.
   */
  public function callGeoService($ip, $ip_service) {
    switch ($ip_service) {
      case 'IPAPI':
        $ipapi_key = $this->config->get('ipapi_key') ?: '';
        $uri = 'https://ipapi.co/' . $ip . '/json?key=' . $ipapi_key;
        break;

      case 'GEOIP':
        $uri = 'http://www.geoplugin.net/json.gp?ip=' . $ip;
        break;

    }

    try {
      $response = $this->client->request('GET', $uri, [
        'timeout' => 5,
        'headers' => ['Accept' => 'application/json'],
      ]);
    }
    catch (GuzzleException $e) {
      $variables = Error::decodeException($e);
      $this->logger->error('%type: @message in %function (line %line of %file).', $variables);
      return [];
    }

    $body = json_decode($response->getBody());

    if ($ip_service == 'GEOIP') {
      $c_code = ($body->geoplugin_continentCode || !empty($body->geoplugin_continentCode)) ? $body->geoplugin_continentCode : 'unavailable';
    } 
    elseif ($ip_service == 'IPAPI') {
      $c_code = ($body->continent_code || !empty($body->continent_code)) ? $body->continent_code : 'unavailable';
    }
    return $c_code;
  }

  /**
   * Determine if useragent is a bot.
   *
   * @param string $user_agent
   *   User agent string.
   *
   * @return bool
   *   True if part of the user agent is a bot identifier value.
   */
  public static function isBot($user_agent) {
    // User lowercase string for comparison.
    $user_agent = strtolower($user_agent);
    // A list of some common words used only for bots and crawlers.
    $bot_identifiers = [
      'bot',
      'slurp',
      'crawler',
      'spider',
      'curl',
      'facebook',
      'fetch',
    ];
    // See if one of the identifiers is in the UA string.
    foreach ($bot_identifiers as $identifier) {
      if (strpos($user_agent, $identifier) !== FALSE) {
        return TRUE;
      }
    }
    return FALSE;
  }

}
